"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FirestoreCollection = void 0;
const admin = __importStar(require("firebase-admin"));
const lodash_chunk_1 = __importDefault(require("lodash.chunk"));
const MAX_BATCH_SIZE = 500;
class FirestoreClient {
    constructor(firestore) {
        this.firestore = firestore;
    }
    getCollection(collectionName, optionalFields = []) {
        return new FirestoreCollection(this.firestore.collection(collectionName), this.firestore, optionalFields);
    }
    runTransaction(updateFunction, transactionOptions) {
        return this.firestore.runTransaction(updateFunction, transactionOptions);
    }
}
exports.default = FirestoreClient;
class FirestoreCollection {
    constructor(collectionReference, firestore, optionalFields) {
        this.collectionReference = collectionReference;
        this.firestore = firestore;
        this.optionalFields = optionalFields;
    }
    async documentExists(id, transaction) {
        const docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        const documentSnapshot = transaction ? await transaction.get(docRef) : await docRef.get();
        if (documentSnapshot === undefined || !documentSnapshot.exists) {
            return Promise.resolve(false);
        }
        return Promise.resolve(true);
    }
    async getDocument(id, transaction) {
        const docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        const documentSnapshot = transaction ? await transaction.get(docRef) : await docRef.get();
        if (documentSnapshot === undefined || !documentSnapshot.exists) {
            return Promise.resolve(undefined);
        }
        return Promise.resolve(this.decode(documentSnapshot));
    }
    async setDocument(id, data, versioned = true) {
        this.deleteOptionalFields(data);
        if (versioned)
            this.applyVersionIncrement(data);
        const docRef = this.collectionReference.doc(id);
        return docRef.set(data, { merge: true });
    }
    async addDocument(data, versioned = true) {
        if (versioned)
            this.applyVersionIncrement(data);
        const docRef = await this.collectionReference.add(data);
        if (docRef === undefined) {
            return Promise.reject();
        }
        return this.getDocument(docRef);
    }
    async deleteDocument(id) {
        return this.collectionReference.doc(id).delete();
    }
    setDocumentInTransaction(id, data, transaction, versioned = true) {
        this.deleteOptionalFields(data);
        if (versioned)
            this.applyVersionIncrement(data);
        const docRef = this.collectionReference.doc(id);
        return transaction.set(docRef, data, { merge: true });
    }
    async executeQuery(fieldPath, opStr, value) {
        const querySnapshot = await this.collectionReference.where(fieldPath, opStr, value).get();
        if (querySnapshot !== undefined) {
            return querySnapshot.docs.map(doc => this.decode(doc));
        }
        return Promise.reject();
    }
    bulkSet(data, versioned = true) {
        Promise.all(lodash_chunk_1.default(data, MAX_BATCH_SIZE).map(async (dataRefs) => {
            const writeBatch = this.firestore.batch();
            (dataRefs).map(value => {
                this.deleteOptionalFields(value);
                if (versioned)
                    this.applyVersionIncrement(value);
                const docRef = value.id ? this.collectionReference.doc(value.id) : this.collectionReference.doc();
                writeBatch.set(docRef, value, { merge: true });
            });
            await writeBatch.commit();
        })).catch(reason => {
            return Promise.reject();
        });
    }
    applyVersionIncrement(data) {
        data.version = admin.firestore.FieldValue.increment(1);
    }
    deleteOptionalFields(data) {
        this.optionalFields
            .filter(prop => data[prop] === undefined)
            .map(value => data[value] = admin.firestore.FieldValue.delete());
    }
    decode(documentSnapshot) {
        return Object.assign(Object.assign({}, documentSnapshot.data()), { created: documentSnapshot.createTime.toDate(), lastModified: documentSnapshot.updateTime.toDate(), id: documentSnapshot.id });
    }
}
exports.FirestoreCollection = FirestoreCollection;
//# sourceMappingURL=FirestoreClient.js.map